@@ -21,11 +21,17 @@ logging.basicConfig(level=logging.DEBUG)
 logging.getLogger('requests.packages.urllib3.connectionpool').setLevel(logging.ERROR)
 
 import os, sys, json, subprocess, argparse
+import locale
+import pipes
 import shutil
 import tempfile
+import time
 from datetime import datetime
 import dxpy, dxpy.app_builder
 
+# Use default locale (used for formatting numbers nicely)
+locale.setlocale(locale.LC_ALL, '')
+
 from dxpy.utils.resolver import resolve_path, is_container_id
 
 parser = argparse.ArgumentParser(description="Uploads a DNAnexus App.")
@@ -36,13 +42,15 @@ parser.add_argument("src_dir", help="App or applet source directory (default: cu
 parser.set_defaults(mode="app")
 parser.add_argument("--create-app", help=argparse.SUPPRESS, action="store_const", dest="mode", const="app")
 parser.add_argument("--create-applet", help=argparse.SUPPRESS, action="store_const", dest="mode", const="applet")
-# TODO: remove -p/--destination-project and deprecation message below.
-parser.add_argument("-p", "--destination-project", help="(Deprecated) Insert the applet into the project with the specified project ID.", default=None)
 parser.add_argument("-d", "--destination", help="Specifies the destination project, destination folder, and/or name for the applet, in the form [PROJECT_NAME_OR_ID:][/[FOLDER/][NAME]]. Overrides the project, folder, and name fields of the dxapp.json, if they were supplied.", default=None)
 
 parser.set_defaults(use_temp_build_project=True)
 parser.add_argument("--no-temp-build-project", help="When building an app, build its applet in the current project instead of a temporary project", action="store_false", dest="use_temp_build_project")
 
+parser.set_defaults(parallel_build=True)
+parser.add_argument("--parallel-build", help=argparse.SUPPRESS, action="store_true", dest="parallel_build")
+parser.add_argument("--no-parallel-build", help="Build with make instead of make -jN.", action="store_false", dest="parallel_build")
+
 # --[no-]publish
 parser.set_defaults(publish=False)
 parser.add_argument("--publish", help="Publish the resulting app and make it the default.", action="store_true", dest="publish")
@@ -53,7 +61,7 @@ parser.set_defaults(remote=False)
 parser.add_argument("--remote", help="Build the app remotely.", action="store_true", dest="remote")
 parser.add_argument("--no-remote", help=argparse.SUPPRESS, action="store_false", dest="remote")
 
-parser.add_argument("-f", "--overwrite", help="If creating an applet, remove existing applets of the same name from the destination project.", action="store_true", default=False)
+parser.add_argument("-f", "--overwrite", help="Remove existing applet(s) of the same name in the destination folder.", action="store_true", default=False)
 parser.add_argument("-v", "--version", help="Override the version number supplied in the manifest.", default=None, dest="version_override", metavar='VERSION')
 parser.add_argument("-b", "--bill-to", help="Entity (of the form user-NAME or org-ORGNAME) to bill for the app.", default=None, dest="bill_to", metavar='USER_OR_ORG')
 
@@ -74,6 +82,22 @@ parser.add_argument("--dx-toolkit-unstable-autodep", help="Auto-insert a dx-tool
 parser.add_argument("--dx-toolkit-autodep", help=argparse.SUPPRESS, action="store_const", dest="dx_toolkit_autodep", const="beta")
 parser.add_argument("--no-dx-toolkit-autodep", help="Do not auto-insert the dx-toolkit dependency if it's absent from the runSpec. See the documentation for more details.", action="store_false", dest="dx_toolkit_autodep")
 
+# --[no-]build (undocumented): perform the ./configure && make step
+parser.set_defaults(build_step=True)
+parser.add_argument("--build-step", help=argparse.SUPPRESS, action="store_true", dest="build_step")
+parser.add_argument("--no-build-step", help=argparse.SUPPRESS, action="store_false", dest="build_step")
+
+# --[no-]upload (undocumented): perform the actual upload
+parser.set_defaults(upload_step=True)
+parser.add_argument("--upload-step", help=argparse.SUPPRESS, action="store_true", dest="upload_step")
+parser.add_argument("--no-upload-step", help=argparse.SUPPRESS, action="store_false", dest="upload_step")
+
+# --[no-]json (undocumented): dumps the JSON describe of the app or
+# --applet that was created
+parser.set_defaults(json=False)
+parser.add_argument("--json", help=argparse.SUPPRESS, action="store_true", dest="json")
+parser.add_argument("--no-json", help=argparse.SUPPRESS, action="store_false", dest="json")
+
 # --[no-]dry-run
 #
 # The --dry-run flag can be used to see the applet spec that would be
@@ -125,12 +149,20 @@ def parse_destination(dest_str):
     # [PROJECT]:/FOLDER/ENTITYNAME
     return resolve_path(dest_str)
 
-def _build_app_remote(src_dir, publish=False, dx_toolkit_autodep="auto"):
+def _build_app_remote(mode, src_dir, destination=None, publish=False,
+                      dx_toolkit_autodep="auto", version_override=None,
+                      bill_to=None, version_autonumbering=True, update=True,
+                      parallel_build=True):
+    if mode == 'app':
+        builder_app = 'app-tarball_app_builder'
+    else:
+        builder_app = 'app-tarball_applet_builder'
+
     temp_dir = tempfile.mkdtemp()
 
     # We have to resolve the correct dx-toolkit dependency type here and
     # explicitly pass it into the interior call of dx-build-app, because
-    # within the execution environment of tarball_app_builder,
+    # within the execution environment of tarball_app(let)_builder,
     # APISERVER_HOST is set to the address of the proxy (a 10.x.x.x
     # address) and doesn't give us any information about whether we are
     # talking to preprod.
@@ -152,46 +184,130 @@ def _build_app_remote(src_dir, publish=False, dx_toolkit_autodep="auto"):
     elif dx_toolkit_autodep == False:
         dx_toolkit_autodep_flag = "--no-dx-toolkit-autodep"
 
-    extra_flags = " ".join([dx_toolkit_autodep_flag])
+    extra_flags = [dx_toolkit_autodep_flag]
+
+    # These flags are basically passed through verbatim.
+    if version_override:
+        extra_flags.extend(['--version', version_override])
+    if bill_to:
+        extra_flags.extend(['--bill-to', bill_to])
+    if not version_autonumbering:
+        extra_flags.append('--no-version-autonumbering')
+    if not update:
+        extra_flags.append('--no-update')
+    if not parallel_build:
+        extra_flags.append('--no-parallel-build')
+
+    using_temp_project_for_remote_build = False
+
+    # If building an applet, run the builder app in the destination
+    # project. If building an app, run the builder app in a temporary
+    # project.
+    dest_folder = None
+    dest_applet_name = None
+    if mode == "applet":
+        # Translate the --destination flag as follows. If --destination
+        # is PROJ:FOLDER/NAME,
+        #
+        # 1. Run the builder app in PROJ
+        # 2. Make the output folder FOLDER
+        # 3. Supply --destination=NAME to the interior call of dx-build-applet.
+        build_project_id = dxpy.WORKSPACE_ID
+        if destination:
+            build_project_id, dest_folder, dest_applet_name = parse_destination(destination)
+        if build_project_id is None:
+            parser.error("Can't create an applet without specifying a destination project; please use the -d/--destination flag to explicitly specify a project")
+        if dest_applet_name:
+            # TODO: escape this correctly, or find a way of passing the
+            # extra_flags that doesn't get screwed up by names that have
+            # spaces in them.
+            extra_flags.extend(['--destination', '/' + dest_applet_name])
+
+    elif mode == "app":
+        using_temp_project_for_remote_build = True
+        build_project_id = dxpy.api.projectNew({"name": "dx-build-app --remote temporary project"})["id"]
 
     try:
         # Resolve relative paths and symlinks here so we have something
         # reasonable to write in the job name below.
         src_dir = os.path.realpath(src_dir)
 
+        # Show the user some progress as the tarball is being generated.
+        # Hopefully this will help them to understand when their tarball
+        # is huge (e.g. the target directory already has a whole bunch
+        # of binaries in it) and interrupt before uploading begins.
         app_tarball_file = os.path.join(temp_dir, "app_tarball.tar.gz")
         # TODO: figure out if we can use --exclude-vcs here (conditional
         # on presence of GNU tar). This might require propagating the
         # --version directly to the interior dx-build-app since in
         # general that can depend on the git metadata.
-        subprocess.check_call(["tar", "-czf", app_tarball_file, "."], cwd=src_dir)
+        tar_subprocess = subprocess.Popen(["tar", "-czf", "-", "."], cwd=src_dir, stdout=subprocess.PIPE)
+        with open(app_tarball_file, 'w') as tar_output_file:
+            total_num_bytes = 0
+            last_console_update = 0
+            start_time = time.time()
+            printed_static_message = False
+            # Pipe the output of tar into the output file, and
+            while True:
+                tar_exitcode = tar_subprocess.poll()
+                data = tar_subprocess.stdout.read(4 * 1024 * 1024)
+                if tar_exitcode is not None and len(data) == 0:
+                    break
+                tar_output_file.write(data)
+                total_num_bytes += len(data)
+                current_time = time.time()
+                # Don't show status messages at all for very short tar
+                # operations (< 1.0 sec)
+                if current_time - last_console_update > 0.25 and current_time - start_time > 1.0:
+                    if sys.stderr.isatty():
+                        if last_console_update > 0:
+                            sys.stderr.write("\r")
+                        sys.stderr.write("Compressing target directory %s... (%s kb)" % (src_dir, locale.format("%d", (total_num_bytes / 1024,), grouping=True),))
+                        sys.stderr.flush()
+                        last_console_update = current_time
+                    elif not printed_static_message:
+                        # Print a message (once only) when stderr is not
+                        # going to a live console
+                        sys.stderr.write("Compressing target directory %s..." % (src_dir,))
+                        printed_static_message = True
 
-        build_project_id = dxpy.api.projectNew({"name": "dx-build-app --remote temporary project"})["id"]
+        if last_console_update > 0:
+            sys.stderr.write("\n")
+        if tar_exitcode != 0:
+            raise Exception("tar exited with non-zero exit code " + str(tar_exitcode))
+
+        dxpy.set_workspace_id(build_project_id)
+
+        remote_file = dxpy.upload_local_file(app_tarball_file, media_type="application/gzip",
+                                             wait_on_close=True, show_progress=True)
 
         try:
-            dxpy.set_workspace_id(build_project_id)
-            remote_file_id = dxpy.upload_local_file(app_tarball_file, media_type="application/gzip",
-                                                    wait_on_close=True, show_progress=True)
-            app_run_result = dxpy.api.appRun(
-                "app-tarball_app_builder",
-                input_params={
-                    "name": "Remote build of %s" % (os.path.basename(src_dir),),
-                    "input": {
-                        "input_file": dxpy.dxlink(remote_file_id),
-                        "publish": publish,
-                        "extra_flags": extra_flags
-                        },
-                    "project": build_project_id
-                    }
-                )
+            extra_flags_str = " ".join(pipes.quote(s) for s in extra_flags)
+            input_hash = {
+                "input_file": dxpy.dxlink(remote_file),
+                "extra_flags": extra_flags_str
+                }
+            if mode == 'app':
+                input_hash["publish"] = publish
+            api_options = {
+                "name": "Remote build of %s" % (os.path.basename(src_dir),),
+                "input": input_hash,
+                "project": build_project_id,
+                }
+            if dest_folder:
+                api_options["folder"] = dest_folder
+            app_run_result = dxpy.api.appRun(builder_app, input_params=api_options)
             job_id = app_run_result["id"]
             print "Started builder job %s" % (job_id,)
             subprocess.check_call(["dx", "watch", job_id])
-
         finally:
-            dxpy.api.projectDestroy(build_project_id)
+            if not using_temp_project_for_remote_build:
+                dxpy.DXProject(build_project_id).remove_objects([remote_file.get_id()])
     finally:
+        if using_temp_project_for_remote_build:
+            dxpy.api.projectDestroy(build_project_id)
         shutil.rmtree(temp_dir)
+
     return
 
 
@@ -214,39 +330,46 @@ def main(**kwargs):
     if not os.path.exists(os.path.join(args.src_dir, "dxapp.json")):
         parser.error("Directory %s does not contain dxapp.json: not a valid DNAnexus app source directory" % args.src_dir)
 
+    if args.mode == "app" and args.destination:
+        parser.error("--destination cannot be used when creating an app (only an applet)")
+
     if args.remote:
-        # To enable these, the tarball builder app needs to learn how to
-        # pass these options through to the interior call of
-        # dx_build_app.
-        if args.mode == 'applet':
-            parser.error('--remote can only be used to create apps')
-        if args.version_override:
-            parser.error('--remote cannot be combined with --version')
-        if args.bill_to:
-            parser.error('--remote cannot be combined with --bill-to')
-        if not args.version_autonumbering:
-            parser.error('--remote cannot be combined with --no-version-autonumbering')
-        if not args.update:
-            parser.error('--remote cannot be combined with --no-update')
+        # The following flags might be useful in conjunction with
+        # --remote. To enable these, we need to learn how to pass these
+        # options through to the interior call of dx_build_app(let).
         if args.dry_run:
             parser.error('--remote cannot be combined with --dry-run')
-        return _build_app_remote(args.src_dir, publish=args.publish, dx_toolkit_autodep=args.dx_toolkit_autodep)
+
+        # The following flags are probably not useful in conjunction
+        # with --remote.
+        if not args.build_step:
+            parser.error('--remote cannot be combined with --no-build-step')
+        if not args.upload_step:
+            parser.error('--remote cannot be combined with --no-upload-step')
+        if args.json:
+            parser.error('--remote cannot be combined with --json')
+        if not args.use_temp_build_project:
+            parser.error('--remote cannot be combined with --no-temp-build-project')
+
+        more_kwargs = {}
+        if args.version_override:
+            more_kwargs['version_override'] = args.version_override
+        if args.bill_to:
+            more_kwargs['bill_to'] = args.bill_to
+        if not args.version_autonumbering:
+            more_kwargs['version_autonumbering'] = False
+        if not args.update:
+            more_kwargs['update'] = False
+        if not args.parallel_build:
+            more_kwargs['parallel_build'] = False
+
+        return _build_app_remote(args.mode, args.src_dir, destination=args.destination, publish=args.publish, dx_toolkit_autodep=args.dx_toolkit_autodep, **more_kwargs)
 
     working_project = None
     using_temp_project = False
     override_folder = None
     override_applet_name = None
 
-    if args.destination_project:
-        if args.destination:
-            # We'll clobber destination below.
-            parser.error("Can't supply both --destination and --destination-project. Just use --destination.")
-        args.destination = args.destination_project
-        print "*"
-        print "* The -p/--destination-project flag has been deprecated. Instead please use"
-        print "* -d=project-XXXX or --destination=project-XXXX, which does the same and more."
-        print "*"
-
     if args.mode == "applet" and args.destination:
         working_project, override_folder, override_applet_name = parse_destination(args.destination)
     elif args.mode == "app" and args.use_temp_build_project and not args.dry_run:
@@ -270,7 +393,11 @@ def main(**kwargs):
                 args.dx_toolkit_autodep = False
             del app_json["buildOptions"]
 
-        dxpy.app_builder.build(args.src_dir)
+        if args.build_step:
+            dxpy.app_builder.build(args.src_dir, parallel_build=args.parallel_build)
+
+        if not args.upload_step:
+            return
 
         bundled_resources = dxpy.app_builder.upload_resources(args.src_dir, project=working_project) if not args.dry_run else []
 
@@ -282,14 +409,16 @@ def main(**kwargs):
                     args.dx_toolkit_autodep = "stable"
                 else:
                     args.dx_toolkit_autodep = "beta"
-            applet_id = dxpy.app_builder.upload_applet(args.src_dir, bundled_resources,
-                                                       check_name_collisions=(args.mode == "applet"),
-                                                       overwrite=args.overwrite and args.mode == "applet",
-                                                       project=working_project,
-                                                       override_folder = override_folder,
-                                                       override_name = override_applet_name,
-                                                       dx_toolkit_autodep=args.dx_toolkit_autodep,
-                                                       dry_run=args.dry_run)
+            applet_id, applet_spec = dxpy.app_builder.upload_applet(
+                args.src_dir,
+                bundled_resources,
+                check_name_collisions=(args.mode == "applet"),
+                overwrite=args.overwrite and args.mode == "applet",
+                project=working_project,
+                override_folder=override_folder,
+                override_name=override_applet_name,
+                dx_toolkit_autodep=args.dx_toolkit_autodep,
+                dry_run=args.dry_run)
         except:
             # Avoid leaking any bundled_resources files we may have
             # created, if applet creation fails. Note that if
@@ -305,6 +434,8 @@ def main(**kwargs):
         if args.dry_run:
             return
 
+        applet_name = applet_spec['name']
+
         print >> sys.stderr, "Created applet " + applet_id + " successfully"
 
         if args.mode == "app":
@@ -315,7 +446,9 @@ def main(**kwargs):
             if not args.version_override and args.version_autonumbering:
                 try_versions.append(version + get_version_suffix(args.src_dir))
 
-            app_id = dxpy.app_builder.create_app(applet_id, args.src_dir,
+            app_id = dxpy.app_builder.create_app(applet_id,
+                                                 applet_name,
+                                                 args.src_dir,
                                                  publish=args.publish,
                                                  set_default=args.publish,
                                                  billTo=args.bill_to,
@@ -323,7 +456,8 @@ def main(**kwargs):
                                                  try_update=args.update)
 
             app_describe = dxpy.api.appDescribe(app_id)
-            print json.dumps(app_describe)
+            if args.json:
+                print json.dumps(app_describe)
 
             if args.publish:
                 print >> sys.stderr, "Uploaded and published app %s/%s (%s) successfully" % (app_describe["name"], app_describe["version"], app_id)
@@ -333,15 +467,16 @@ def main(**kwargs):
                 print >> sys.stderr, "  dx api app-%s/%s publish \"{\\\"makeDefault\\\": true}\"" % (app_describe["name"], app_describe["version"])
 
         elif args.mode == "applet":
-            pass
+            if args.json:
+                print json.dumps(dxpy.api.appletDescribe(applet_id))
         else:
-            raise ValueError("Unrecognized mode %r" % (args.mode,))
+            raise dxpy.app_builder.AppBuilderException("Unrecognized mode %r" % (args.mode,))
 
-    except dxpy.app_builder.AppletBuilderException as e:
-        # AppletBuilderException represents errors during applet
-        # building that could reasonably have been anticipated.
-        print "Error: %s" % (e.message,)
-        sys.exit(1)
+    except dxpy.app_builder.AppBuilderException as e:
+        # AppBuilderException represents errors during app or applet building
+        # that could reasonably have been anticipated by the user.
+        print >> sys.stderr, "Error: %s" % (e.message,)
+        sys.exit(3)
 
     finally:
         # Clean up after ourselves.
