@@ -207,15 +207,19 @@ def print_list_field(label, values):
 def print_json_field(label, json_value):
     print_field(label, json.dumps(json_value, ensure_ascii=False))
 
-def print_project_desc(desc):
-    recognized_fields = ['id', 'class', 'name', 'description', 'protected', 'restricted', 'created', 'modified', 'dataUsage', 'sponsoredDataUsage', 'tags', 'level', 'folders', 'objects', 'permissions', 'properties', 'appCaches', 'billTo']
+def print_project_desc(desc, verbose=False):
+    recognized_fields = ['id', 'class', 'name', 'summary', 'description', 'protected', 'restricted', 'created', 'modified', 'dataUsage', 'sponsoredDataUsage', 'tags', 'level', 'folders', 'objects', 'permissions', 'properties', 'appCaches', 'billTo', 'version']
 
     print_field("ID", desc["id"])
     print_field("Class", desc["class"])
     if "name" in desc:
         print_field("Name", desc["name"])
-    if 'description' in desc:
-        print_field("Description", desc["description"])
+    if 'summary' in desc:
+        print_field("Summary", desc["summary"])
+    if 'description' in desc and (verbose or 'summary' not in desc):
+        print_field("Description", desc['description'])
+    if 'version' in desc and verbose:
+        print_field("Version", str(desc['version']))
     if 'billTo' in desc:
         print_field("Billed to",  desc['billTo'][5 if desc['billTo'].startswith('user-') else 0:])
     if 'protected' in desc:
@@ -374,7 +378,7 @@ def print_data_obj_desc(desc, verbose=False):
                 if desc["class"] == "file" or desc["class"] == "gtable":
                     sponsored_str = ""
                     if 'sponsored' in desc and desc['sponsored']:
-                        sponsored_str = ", sponsored by DNAnexus"
+                        sponsored_str = DELIMITER(", ") + "sponsored by DNAnexus"
                     print_field("Size", get_size_str(desc['size']) + sponsored_str)
                 else:
                     print_field("Size", str(desc['size']))
@@ -497,7 +501,7 @@ def print_desc(desc, verbose=False):
     formatted and human-readable string containing the data in *desc*.
     '''
     if desc['class'] in ['project', 'workspace', 'container']:
-        print_project_desc(desc)
+        print_project_desc(desc, verbose=verbose)
     elif desc['class'] == 'app':
         print_app_desc(desc, verbose=verbose)
     elif desc['class'] == 'job':
@@ -558,16 +562,21 @@ def get_find_jobs_string(jobdesc, has_children, single_result=False):
     :param single_result: whether the job is displayed as a single result or as part of a job tree
     '''
     is_origin_job = jobdesc['parentJob'] is None or single_result
-    string = ("* " if is_origin_job and get_delimiter() is None else "")
-    string += (BOLD() + BLUE() + (jobdesc['name'] if 'name' in jobdesc else "<no name>") + ENDC()) + DELIMITER(' (') + JOB_STATES(jobdesc['state']) + DELIMITER(') ') + jobdesc['id'] 
-    string += DELIMITER('\n' + (u'│ ' if is_origin_job and has_children else ("  " if is_origin_job else "")))
-    string += jobdesc['launchedBy'][5:] + DELIMITER(' ')
-    string += str(datetime.datetime.fromtimestamp(jobdesc['created']/1000))
+    result = ("* " if is_origin_job and get_delimiter() is None else "")
+    canonical_job_name = jobdesc['executableName'] + ":" + jobdesc['function']
+    job_name = jobdesc.get('name', '<no name>')
+    result += BOLD() + BLUE() + job_name + ENDC()
+    if job_name != canonical_job_name and job_name+":main" != canonical_job_name:
+        result += ' (' + canonical_job_name + ')'    
+    result += DELIMITER(' (') + JOB_STATES(jobdesc['state']) + DELIMITER(') ') + jobdesc['id']
+    result += DELIMITER('\n' + (u'│ ' if is_origin_job and has_children else ("  " if is_origin_job else "")))
+    result += jobdesc['launchedBy'][5:] + DELIMITER(' ')
+    result += str(datetime.datetime.fromtimestamp(jobdesc['created']/1000))
     if jobdesc['state'] in ['done', 'failed', 'terminated', 'waiting_on_output']:
         # TODO: Remove this check once all jobs are migrated to have these values
         if 'stoppedRunning' in jobdesc and 'startedRunning' in jobdesc:
-            string += " (runtime {r})".format(r=str(datetime.timedelta(seconds=int(jobdesc['stoppedRunning']-jobdesc['startedRunning'])/1000)))
+            result += " (runtime {r})".format(r=str(datetime.timedelta(seconds=int(jobdesc['stoppedRunning']-jobdesc['startedRunning'])/1000)))
     elif jobdesc['state'] == 'running':
-        string += " (running for {rt})".format(rt=datetime.timedelta(seconds=int(time.time()-jobdesc['startedRunning']/1000)))
+        result += " (running for {rt})".format(rt=datetime.timedelta(seconds=int(time.time()-jobdesc['startedRunning']/1000)))
 
-    return string
+    return result
